@@ -88,12 +88,7 @@ extern int catch_exceptions_with_msg (struct ui_out *uiout,
 typedef int (catch_errors_ftype) (void *);
 extern int catch_errors (catch_errors_ftype *, void *, char *, return_mask);
 
-#ifndef __cplusplus
-# define GDB_NOEXCEPT
-#elif __cplusplus < 201103L
-# define GDB_NOEXCEPT throw ()
-#else
-# define GDB_NOEXCEPT noexcept
-#endif
-
+/* Compare two exception objects for print equality.  */
+extern int exception_print_same (struct gdb_exception e1,
+				 struct gdb_exception e2);
 #endif
