@@ -1,6 +1,6 @@
 /* Pascal language support definitions for GDB, the GNU debugger.
 
-   Copyright (C) 2000-2016 Free Software Foundation, Inc.
+   Copyright (C) 2000-2017 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -36,9 +36,9 @@ extern void pascal_print_type (struct type *, const char *, struct ui_file *,
 extern void pascal_print_typedef (struct type *, struct symbol *,
 				  struct ui_file *);
 
-extern void pascal_val_print (struct type *, const gdb_byte *, int,
+extern void pascal_val_print (struct type *, int,
 			      CORE_ADDR, struct ui_file *, int,
-			      const struct value *,
+			      struct value *,
 			      const struct value_print_options *);
 
 extern void pascal_value_print (struct value *, struct ui_file *,
@@ -75,7 +75,7 @@ extern void pascal_object_print_value_fields (struct type *, const gdb_byte *,
 					      LONGEST,
 					      CORE_ADDR, struct ui_file *,
 					      int,
-					      const struct value *,
+					      struct value *,
 					      const struct value_print_options *,
 					      struct type **, int);
 
