@@ -1,6 +1,6 @@
 /* Python interface to architecture
 
-   Copyright (C) 2013-2016 Free Software Foundation, Inc.
+   Copyright (C) 2013-2017 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -22,6 +22,7 @@
 #include "arch-utils.h"
 #include "disasm.h"
 #include "python-internal.h"
+#include "py-ref.h"
 
 typedef struct arch_object_type_object {
   PyObject_HEAD
@@ -120,7 +121,7 @@ archpy_disassemble (PyObject *self, PyObject *args, PyObject *kw)
   CORE_ADDR pc;
   gdb_py_ulongest start_temp;
   long count = 0, i;
-  PyObject *result_list, *end_obj = NULL, *count_obj = NULL;
+  PyObject *end_obj = NULL, *count_obj = NULL;
   struct gdbarch *gdbarch = NULL;
 
   ARCHPY_REQUIRE_VALID (self, gdbarch);
@@ -141,14 +142,14 @@ archpy_disassemble (PyObject *self, PyObject *args, PyObject *kw)
 	 conversion process.  */
       if (PyLong_Check (end_obj))
         end = PyLong_AsUnsignedLongLong (end_obj);
+#if PY_MAJOR_VERSION == 2
       else if (PyInt_Check (end_obj))
         /* If the end_pc value is specified without a trailing 'L', end_obj will
            be an integer and not a long integer.  */
         end = PyInt_AsLong (end_obj);
+#endif
       else
         {
-          Py_DECREF (end_obj);
-          Py_XDECREF (count_obj);
           PyErr_SetString (PyExc_TypeError,
                            _("Argument 'end_pc' should be a (long) integer."));
 
@@ -157,8 +158,6 @@ archpy_disassemble (PyObject *self, PyObject *args, PyObject *kw)
 
       if (end < start)
         {
-          Py_DECREF (end_obj);
-          Py_XDECREF (count_obj);
           PyErr_SetString (PyExc_ValueError,
                            _("Argument 'end_pc' should be greater than or "
                              "equal to the argument 'start_pc'."));
@@ -171,8 +170,6 @@ archpy_disassemble (PyObject *self, PyObject *args, PyObject *kw)
       count = PyInt_AsLong (count_obj);
       if (PyErr_Occurred () || count < 0)
         {
-          Py_DECREF (count_obj);
-          Py_XDECREF (end_obj);
           PyErr_SetString (PyExc_TypeError,
                            _("Argument 'count' should be an non-negative "
                              "integer."));
@@ -181,7 +178,7 @@ archpy_disassemble (PyObject *self, PyObject *args, PyObject *kw)
         }
     }
 
-  result_list = PyList_New (0);
+  gdbpy_ref<> result_list (PyList_New (0));
   if (result_list == NULL)
     return NULL;
 
@@ -196,63 +193,41 @@ archpy_disassemble (PyObject *self, PyObject *args, PyObject *kw)
        || (end_obj == NULL && count_obj == NULL && pc == start);)
     {
       int insn_len = 0;
-      char *as = NULL;
-      struct ui_file *memfile = mem_fileopen ();
-      PyObject *insn_dict = PyDict_New ();
+      gdbpy_ref<> insn_dict (PyDict_New ());
 
       if (insn_dict == NULL)
-        {
-          Py_DECREF (result_list);
-          ui_file_delete (memfile);
+	return NULL;
+      if (PyList_Append (result_list.get (), insn_dict.get ()))
+	return NULL;  /* PyList_Append Sets the exception.  */
 
-          return NULL;
-        }
-      if (PyList_Append (result_list, insn_dict))
-        {
-          Py_DECREF (result_list);
-          Py_DECREF (insn_dict);
-          ui_file_delete (memfile);
-
-          return NULL;  /* PyList_Append Sets the exception.  */
-        }
+      string_file stb;
 
       TRY
         {
-          insn_len = gdb_print_insn (gdbarch, pc, memfile, NULL);
+          insn_len = gdb_print_insn (gdbarch, pc, &stb, NULL);
         }
       CATCH (except, RETURN_MASK_ALL)
         {
-          Py_DECREF (result_list);
-          ui_file_delete (memfile);
-
 	  gdbpy_convert_exception (except);
 	  return NULL;
         }
       END_CATCH
 
-      as = ui_file_xstrdup (memfile, NULL);
-      if (PyDict_SetItemString (insn_dict, "addr",
+      if (PyDict_SetItemString (insn_dict.get (), "addr",
                                 gdb_py_long_from_ulongest (pc))
-          || PyDict_SetItemString (insn_dict, "asm",
-                                   PyString_FromString (*as ? as : "<unknown>"))
-          || PyDict_SetItemString (insn_dict, "length",
+          || PyDict_SetItemString (insn_dict.get (), "asm",
+                                   PyString_FromString (!stb.empty ()
+							? stb.c_str ()
+							: "<unknown>"))
+          || PyDict_SetItemString (insn_dict.get (), "length",
                                    PyInt_FromLong (insn_len)))
-        {
-          Py_DECREF (result_list);
-
-          ui_file_delete (memfile);
-          xfree (as);
-
-          return NULL;
-        }
+	return NULL;
 
       pc += insn_len;
       i++;
-      ui_file_delete (memfile);
-      xfree (as);
     }
 
-  return result_list;
+  return result_list.release ();
 }
 
 /* Initializes the Architecture class in the gdb module.  */
